#!/bin/bash 

set -e

PWD=$(pwd)
APTDIR=/etc/apt
SOURCESDIR=/etc/apt/sources.list.d
NEWAPTDIR=$(pwd)/apt
NEWSOURCESDIR=$(pwd)/sources.list.d
RED='\033[1;31m'
GREEN='\033[0;32m'
BGREEN='\033[1;32m'
BLUE='\033[1;34m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color
ALERT=80

clear
echo -e ""
echo -e "${BLUE}       Bienvenidos/as al proceso de actualización de Huayra 6.5 - Ráfaga ${NC}"
echo -e ""
echo -e "${BGREEN}    Este proceso actualizará el equipo a la última versión de Huayra/Linux ${NC}"
echo -e ""
echo -e "${YELLOW}                               A TENER EN CUENTA !"
echo -e ""
echo -e "${YELLOW} Si instalaste otras aplicaciones como Chrome, Zoom, etc que no están incluidas${NC}"
echo -e "${YELLOW}   en la instalación original de Huayra, deberás reactivar sus repositorios ${NC}"
echo -e "${YELLOW}      a continuación de esta actualización, ya que serán deshabilitados. ${NC}"
echo -e ""
echo -e "${RED}                                   ATENCION !"
echo -e ""
echo -e "${RED}  Como tomará tiempo realizarlo, el equipo debe estar conectado a la corriente${NC}"
echo -e "${RED} eléctrica y así evitar inconvenientes ante una eventual descarga de la batería${NC}"
echo -e ""

read -p $'\033[1;32mESTA SEGURO DE CONTINUAR CON EL PROCESO (s/n)?: ' -e key;
if [ $key != "s" ]; then
  echo -e "${RED}Proceso cancelado ${NC}"
  exit 1
fi

## Make sure we're enough space to upgrade
echo -e "${BLUE}Chequeando si hay suficiente espacio en disco para actualizar... ${NC}"
df -H / | grep -vE '^Filesystem|fichero|uso|tmpfs|cdrom|loop|udev' | awk '{ print $5 " " $1 " " $6 }' | while read -r output;
do
  usep=$(echo "$output" | awk '{ print $1}' | cut -d'%' -f1 )
  partition=$(echo "$output" | awk '{ print $2 }' )
  if [ $usep -ge $ALERT ]; then
    echo -e "${RED}No hay suficiente espacio para actualizar \"$partition ($usep%)\" en $(hostname) ${NC}"
    echo -e "${RED}Proceso cancelado ${NC}"
    exit 1
  fi
done
echo -e "${GREEN}Verificado OK. ${NC}"

## Make sure we're in the sources_dir directory
echo -e "${BLUE}Chequeando que se puede acceder al directorio APT... ${NC}"
cd $APTDIR
echo -e "${GREEN}Hecho ${NC}" 

## Make sure we're in the sources_dir directory
echo -e "${BLUE}Chequeando que se puede acceder al directorio de otros origenes de datos de APT... ${NC}"
cd $SOURCESDIR
echo -e "${GREEN}Hecho ${NC}" 

##  Now rename sources.list file to disabled
echo -e "${BLUE}Renombrando como disabled el origen de datos de APT... ${NC}"
if [ -f $APTDIR/sources.list ]; then
  mv $APTDIR/sources.list $APTDIR/sources.list.disabled; 
fi
echo -e "${GREEN}Renombrados ${NC}" 

##  Now copy new sources.list file
echo -e "${BLUE}Copiando el nuevo origen de datos de APT... ${NC}" 
cp $NEWAPTDIR/sources.list $APTDIR/; 
echo -e "${GREEN}Copiado ${NC}" 

##  See if we found any list file
echo -e "${BLUE}Verificando archivos deshabilitados en el directorio de otros origenes de datos de APT... ${NC}"
found=0
for i in $SOURCESDIR/*.list.disabled; do
  if [ -f $i ]; then
    found=`expr $found + 1`
  fi
done
echo -e "${GREEN}Verificados ${NC}" 

##  If we found none then continue
if [ "$found" -lt 1 ]; then
    ##  See if we found any list file
    echo -e "${BLUE}}Verificando archivos en el directorio de otros origenes de datos de APT... ${NC}"
    found=0
    for i in $SOURCESDIR/*.list; do
        if [ -e $i ]; then
            found=`expr $found + 1`
        fi
    done

    ##  If we found none then exit
    if [ "$found" -lt 1 ]; then
        echo -e "${RED}No se encontraron archivos. Verifique que utiliza privilegios de supervisor (sudo) para ejecutar la actualizacion ${NC}"
        exit 1
    fi
    echo -e "${GREEN}Encontrados ${NC}" 
    ##  Now rename every list file to disabled
    echo -e "${BLUE}Renombrando archivos anteriores del directorio de otros origenes de datos de APT... ${NC}"
    for i in $SOURCESDIR/*.list; do
        ## Rename files.
        echo -e "${BLUE}Renombrando " $i " como " $i.disabled " ${NC}"
        mv $i $i.disabled; 
    done
    echo -e "${GREEN}Renombrados ${NC}" 
else
  echo -e "${YELLOW}Se encontraron archivos deshabilitados, se evita renombrar nuevamente. ${NC}"
fi

##  Now copy file to sources_dir
echo -e "${BLUE}Copiando archivo con los nuevos repositorios al directorio de otros origenes de datos de APT... ${NC}" 
cp -f $NEWSOURCESDIR/huayra.list $SOURCESDIR/; 
echo -e "${GREEN}Copiado ${NC}"  

##  See if we could copy list files
echo -e "${BLUE}Verificando archivos nuevos en el directorio de otros origenes de datos de APT... ${NC}"
found=0
for i in $SOURCESDIR/*.list; do
  if [ -f $i ]; then
    found=`expr $found + 1`
  fi
done

##  If we found none then exit
if [ "$found" -lt 1 ]; then
  echo -e "${RED}No se pudieron copiar los archivos. Verifique que utiliza privilegios de supervisor (sudo) para ejecutar la actualizacion ${NC}" 
  exit 1
fi
echo -e "${GREEN}Verificado ${NC}" 

export NEEDRESTART_MODE=a
export DEBIAN_FRONTEND=noninteractive
export DEBIAN_PRIORITY=critical

echo -e ""
echo -e ""
echo -e "${BLUE} COMENZARÁ EL PROCESO DE ACTUALIZACION ${NC}"
echo -e ""
echo -e ""
read -sp $'\033[1;32mPulse ENTER para continuar...'
echo -e ""

## Trying to apt clean 
echo -e "${BLUE}Intentando limpiar APT cache ${NC}"
apt -qy clean
echo -e "${GREEN}Completado ${NC}" 

## Trying to update huayra and debian
echo -e "${BLUE}Intentando descargar la lista de paquetes para actualizar ${NC}"
apt -qy update
echo -e "${GREEN}Lista descargada ${NC}" 

## Trying to download upgradables packages huayra and debian
echo -e "${BLUE}Descargando paquetes para actualizar ${NC}"
apt -d -qy -o "Dpkg::Options::=--force-confdef" -o "Dpkg::Options::=--force-confold" dist-upgrade
echo -e "${GREEN}Paquetes descargados ${NC}" 

## Trying to upgrade huayra and debian
echo -e "${BLUE}Instalando nuevos paquetes ${NC}"
apt -qy -o "Dpkg::Options::=--force-confdef" -o "Dpkg::Options::=--force-confold" dist-upgrade
echo -e "${GREEN}Instalacion concluida ${NC}" 

## Trying to update grub
echo -e "${BLUE}Actualizando grub ${NC}"
update-grub
echo -e "${GREEN}Actualización concluida ${NC}" 
